module Page.Landing exposing
    ( Model
    , Msg
    , view
    , init
    , update
    , subscriptions
    )
import Element exposing (Element)
import Websockets
import Ports
import Json.Decode as Decode
import Json.Encode as Encode exposing (Value)
import Html.Attributes exposing (placeholder)
import Element.Input
import Element.Background

import EncodeDecode

type alias Model = {
    time : String,
    greetWidgetText : String,
    greeting : String
    }

type Msg
    = DownMsg EncodeDecode.DownMsg
    | UpMsg EncodeDecode.LandingUpMsg
    | DecodeError Decode.Error
    | GreetWidgetText String
    | NoOp

init : () -> (Model, Cmd Msg)
init flags = (
    { time = "time not yet set"
    , greetWidgetText = ""
    , greeting = ""
    },
    Cmd.none
    )

subscriptions : Model -> Sub Msg
subscriptions model =
    Ports.socketOnEvent
        (Websockets.EventHandlers
            (\_ -> NoOp)
            (\_ -> NoOp)
            (\_ -> NoOp)
            (\message ->
                case Decode.decodeString EncodeDecode.downMsgDecoder message.data of
                    Ok msg -> DownMsg msg
                    Err err -> DecodeError err
            )
            (\_ -> NoOp)
        )

update : Msg -> Model -> (Model, Cmd Msg)
update msg model =
    case msg of
        DownMsg (EncodeDecode.LandingDownMsg (EncodeDecode.TimeUpdate time)) ->
            ( {model | time = time}, Cmd.none )
        DownMsg (EncodeDecode.LandingDownMsg (EncodeDecode.Greeting greeting)) ->
            ( {model | greeting = greeting}, Cmd.none)
        UpMsg upMsg ->
            let cmd = Ports.socketSend <| EncodeDecode.upMsgEncoder <| EncodeDecode.LandingUpMsg upMsg
            in
                (model, cmd)

        GreetWidgetText text -> ( {model | greetWidgetText = text}, Cmd.none )
        _ -> (model, Cmd.none)

greetWidget : Model -> Element Msg
greetWidget model =
    let
        idleBlue = Element.rgb255 18 147 217
        focusBlue = Element.rgb255 18 125 184

        myButton = Element.Input.button
            [ Element.Background.color idleBlue
            , Element.mouseOver [Element.Background.color focusBlue]
            , Element.width (Element.fill |> Element.maximum 100)
            , Element.height (Element.fill |> Element.maximum 50)
            ]
            { onPress = Just <| UpMsg <| EncodeDecode.RequestGreet model.greetWidgetText
            , label = Element.text "Greet"
            }
        placeholder  = case model.greetWidgetText of
            "" -> Just <| Element.Input.placeholder [] <| Element.text "Type Your Name"
            _ -> Just <| Element.Input.placeholder [] <| Element.text ""
        textInput =
            Element.Input.text
            [ Element.width (Element.fill |> Element.maximum 400)
            , Element.height Element.fill
            ]
                { onChange = GreetWidgetText
                , text = model.greetWidgetText
                , placeholder = placeholder
                , label = Element.Input.labelHidden "Enter your name"
                }
        nameInput = Element.row [] [ textInput , myButton]
    in
        Element.column []
            [ nameInput
            , Element.text model.greeting
            ]

view : Model -> Element Msg
view model =
    Element.column []
        [ Element.text "Landing"
        , Element.text <| "Current time is : " ++ model.time
        , greetWidget model
        ]
